/***************************************************************************
                          modemcomms.cpp  -  description
                             -------------------
    begin                : Sat Dec 22 2001
    copyright            : (C) 2001 by Robin Atwood
    email                : robin.atwood@attglobal.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "modemcomms.h"

using namespace std;

ModemComms::ModemComms( const char _port[] )
{
	struct stat stat_p;
  if ( stat( _port, &stat_p) > 0 )
	{
		cerr << "device" << _port << " does not exist" << endl;
		exit( -1 );
	}
	strcpy( port, _port );
	locked = false;
}
		
int ModemComms::initPort()
{
	/* This code was adapted from the Serial Programming HOWTO. */
	
	fd = open( port, O_RDWR | O_NDELAY );
	if ( fd < 0 )
	{
	  return 1;
	}

	if ( tcgetattr( fd, &oldtio ) < 0 )
	{
		close( fd );
		return 2;
	}

	bzero( &newtio, sizeof(newtio) ); 						/* clear struct for new port settings */

	/*
	BAUDRATE: Set bps rate. You could also use cfsetispeed and cfsetospeed.
	CRTSCTS : output hardware flow control (only used if the cable has
		all necessary lines. See sect. 7 of Serial-HOWTO)
	CS8     : 8n1 (8bit,no parity,1 stopbit)
	CLOCAL  : local connection, no modem contol
	CREAD   : enable receiving characters
	*/
	newtio.c_cflag = BAUDRATE | CRTSCTS | CS8 | CREAD | CLOCAL;

	/*
	IGNPAR  : ignore bytes with parity errors
	ICRNL   : map CR to NL (otherwise a CR input on the other computer
		will not terminate input)
	otherwise make device raw (no other input processing)
	*/
	newtio.c_iflag = ICRNL | BRKINT;

	/*
	Raw output.
	*/
	newtio.c_oflag = OPOST | ONLCR;

	/*
	ICANON  : enable canonical input
	disable all echo functionality, and don't send signals to calling program
	*/
	newtio.c_lflag = ICANON;

	/*
	initialize all control characters
	default values can be found in /usr/include/termios.h, and are given
	in the comments, but we don't need them here
	*/
	newtio.c_cc[VEOF]     = 4;     /* Ctrl-d */
	newtio.c_cc[VTIME]    = 0;     /* inter-character timer unused */
	newtio.c_cc[VMIN]     = 1;     /* blocking read until 1 character arrives */

	/*
	now clean the modem line and activate the settings for the port
	*/
	tcdrain( fd );
	tcflush( fd, TCIOFLUSH );
	tcsetattr( fd, TCSANOW, &newtio );
	return 0;
};

void ModemComms::readPort()
{
	bzero( buf, 255 );
	int l = read( fd, buf, 255 );
	buf[l-1] = '\0';             /* set end of string */
}

void ModemComms::writePort( const char cmd[] )
{
	int l = strlen( cmd );
	strcpy( buf, cmd );
	buf[l] = '\n';
	write( fd, buf, l+1 );
}

void ModemComms::closePort()
{
	/* restore the old port settings */
	tcsetattr( fd, TCSANOW, &oldtio );
	close( fd );
}

bool ModemComms::modemIO( const char atcmd[] )
{
	/* The modem seems to respond to a command with either an echo or an OK.   */
	/* We check for all these possibilities until I understand serial port     */
	/* programming a bit better!                                               */

	int k;
	cout << "command out (" << atcmd << ")" << endl;
	writePort( atcmd );
	for ( k = 0; k < 20; ++k )
	{
  	readPort();
		if ( strlen( buf ) > 1 ) break;  /* ignore nulls and NL */
		usleep( 200000 );
  }
	cout << "command in  (" << buf   << ")" << endl;

	return ( strcmp( atcmd, buf ) == 0 || strcmp( "OK", buf ) == 0 );
}

bool ModemComms::lockDevice()
{
	int fd; char *p; char pidbuf[11];
	pid_t pid;
	
	p = strrchr( port, '/' );
	if ( p == NULL )
		return false;   										// port is not a /dev/xxx
	++p;
	strcpy( lockfile, LOCKDIR );
	strcat( lockfile, (const char *) p );
	
	fd = open( lockfile, O_WRONLY | O_EXCL | O_CREAT, 0666 );
	if ( fd < 0 )
	{
		fd = open( lockfile, O_RDONLY );
		if ( fd < 0 ) return false;
		read( fd, pidbuf, 11 );
		pidbuf[10] = '\0';
		sscanf(pidbuf, "%d", &pid );
		close( fd );
		if ( kill( pid, 0 ) == 0 )					// lock file still owned
			return false;
		unlink( lockfile );									// delete stale lock file
		cerr << "Purging stale lock file " << lockfile << endl;
		fd = open( lockfile, O_WRONLY | O_EXCL | O_CREAT, 0666 );
		if ( fd < 0 )												// still cannot make lock file
			return false;											// give up
	}
	
	sprintf( pidbuf, "%10d\n", getpid() );
	write( fd, pidbuf, 11 );
	close( fd );
	locked = true;
	return true;
}

bool ModemComms::unlockDevice()
{
	if ( !locked ) return false;		
	unlink( lockfile );	
	locked = false;
	return true;
}

char * ModemComms::getResp()
{
	// FIXME! I never see BUSY here
	readPort();
  int k = 0, l = strlen(buf);
  while ( k < l )
  {
    if ( buf[k] == '\n' ) buf[k] = ' ';
    k = k + 1;
  }
	cout << "response in  (" << buf   << ")" << endl;
	return &buf[0];	
}

void ModemComms::setICanon( bool on )
{
  if ( on )
  {
   	newtio.c_lflag = ICANON;
  	newtio.c_cc[VTIME]    = 0;     /* inter-character timer unused */
  	newtio.c_cc[VMIN]     = 1;     /* blocking read until 1 character arrives */
  }
  else
  {
  	newtio.c_lflag = ~ICANON;
  	newtio.c_cc[VTIME]    = 4;     /* inter-character timer */
  	newtio.c_cc[VMIN]     = 4;     /* blocking read until 4 characters arrive */
  }

	tcflush( fd, TCIOFLUSH );
	tcsetattr( fd, TCSANOW, &newtio );
}
